#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from collections import OrderedDict

from persistent import Persistent
from zope.componentvocabulary.vocabulary import UtilityTerm, UtilityVocabulary
from zope.schema.fieldproperty import FieldProperty

from pyams_content.shared.form.interfaces import IFormHandler, IMailtoHandlerInfo, \
    IMailtoHandlerTarget
from pyams_utils.adapter import adapter_config, get_adapter_weight, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import utility_config
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'

from pyams_content import _


@vocabulary_config(name='PyAMS form handlers')
class FormHandlersVocabulary(UtilityVocabulary):
    """Form handlers vocabulary"""

    interface = IFormHandler

    def __init__(self, context, **kw):
        request = check_request()
        registry = request.registry
        translate = request.localizer.translate
        utils = [(None, translate(_("No selected handler...")))] + \
                [(name, translate(util.label))
                 for (name, util) in sorted(registry.getUtilitiesFor(self.interface),
                                            key=get_adapter_weight)]
        self._terms = OrderedDict((title, UtilityTerm(name, title)) for name, title in utils)

    def __iter__(self):
        return iter(self._terms.values())


#
# Mailto form handler
#

MAILTO_HANDLER_ANNOTATIONS_KEY = 'pyams_content.form.handler.mailto'


@factory_config(IMailtoHandlerInfo)
class MailtoFormHandlerInfo(Persistent):
    """Mailto form handler persistent info"""

    source_address = FieldProperty(IMailtoHandlerInfo['source_address'])
    source_name = FieldProperty(IMailtoHandlerInfo['source_name'])
    target_address = FieldProperty(IMailtoHandlerInfo['target_address'])
    target_name = FieldProperty(IMailtoHandlerInfo['target_name'])


@adapter_config(context=IMailtoHandlerTarget, provides=IMailtoHandlerInfo)
def mailto_form_handler_factory(context):
    """Mailto form handler factory"""
    return get_annotation_adapter(context, MAILTO_HANDLER_ANNOTATIONS_KEY, IMailtoHandlerInfo)


@utility_config(name='mailto', provides=IFormHandler)
class MailtoFormHandler(object):
    """Mailto form handler"""

    label = _("Direct mailto form handler")
    weight = 10

    target_interface = IMailtoHandlerTarget
    handler_info = IMailtoHandlerInfo

    def handle(self, form, data, user_data):
        # TODO: handle form data
        pass
